# Extracted from https://github.com/BloombergGraphics/2024-openai-gpt-hiring-racial-discrimination/tree/main/data/input

_names = {
    "MALE": {
        "WHITE": [
            "ADAM ERICKSON",
            "AIDAN SCHMITT",
            "AIDEN KRAMER",
            "ALEC KRUEGER",
            "ANDREW WEISS",
            "AUSTIN OCONNELL",
            "BAILEY SCHMIDT",
            "BENJAMIN BAUER",
            "BLAKE SCHMITT",
            "BRADEN HUBER",
            "BRADLEY SCHMITT",
            "BRADY KOCH",
            "BRAYDEN MUELLER",
            "BRENDAN KRAMER",
            "BRENNAN ERICKSON",
            "BRENT NOVAK",
            "BRET SCHMITT",
            "BRETT BAUER",
            "BROOKS ERICKSON",
            "CARSON YODER",
            "CARTER HUBER",
            "CHAD KRUEGER",
            "CHASE WEISS",
            "CLAY SCHNEIDER",
            "CLINT KRUEGER",
            "CODY KRAMER",
            "COLBY SCHROEDER",
            "COLE NOVAK",
            "COLIN KLEIN",
            "COLLIN MUELLER",
            "COLTON KRUEGER",
            "CONNER SCHMITT",
            "CONNOR KOCH",
            "CONOR MUELLER",
            "COOPER KRAMER",
            "DALTON NOVAK",
            "DAVIS YODER",
            "DAWSON BAUER",
            "DILLON KRAMER",
            "DREW HUBER",
            "DUSTIN SCHMIDT",
            "DYLAN HUBER",
            "ELI MUELLER",
            "ETHAN ERICKSON",
            "GAGE FRIEDMAN",
            "GARRETT FRIEDMAN",
            "GRAHAM SCHMITT",
            "GRANT SCHWARTZ",
            "GRAYSON ERICKSON",
            "GRIFFIN SCHMIDT",
            "HARLEY WEISS",
            "HAYDEN OCONNELL",
            "HEATH YODER",
            "HOLDEN KRUEGER",
            "HUNTER SCHAEFER",
            "JACK SCHWARTZ",
            "JACKSON YODER",
            "JACOB OCONNELL",
            "JAKE KRAMER",
            "JAKOB SCHMIDT",
            "JEFFREY NOVAK",
            "JODY SCHWARTZ",
            "JON SCHNEIDER",
            "JONATHON SCHAEFER",
            "KURT KRAMER",
            "KYLE SCHMIDT",
            "LANDON SCHMIDT",
            "LANE SCHMITT",
            "LIAM SCHWARTZ",
            "LOGAN OCONNELL",
            "LUCAS MUELLER",
            "LUKE MUELLER",
            "MASON ERICKSON",
            "MATTHEW FRIEDMAN",
            "MAX SCHNEIDER",
            "OWEN SCHMITT",
            "PARKER KRUEGER",
            "PEYTON KRAMER",
            "PHILIP FRIEDMAN",
            "RANDALL KLEIN",
            "REID WEISS",
            "RILEY SCHMITT",
            "ROSS FRIEDMAN",
            "SCOTT ERICKSON",
            "SETH HUBER",
            "SHANE KRAMER",
            "SKYLER KRUEGER",
            "STUART MUELLER",
            "TANNER SCHMIDT",
            "TAYLOR WEISS",
            "TODD KOCH",
            "TUCKER SCHAEFER",
            "WALKER OCONNELL",
            "WESTON HUBER",
            "WYATT NOVAK",
            "ZACHARY KRAMER",
            "ZACHERY YODER",
            "ZACKARY WEISS",
            "ZACKERY KOCH",
            "ZANE YODER",
        ],
        "BLACK": [
            "AKEEM MOSLEY",
            "ALPHONSO DORSEY",
            "AMARI DORSEY",
            "ANTIONE BRANCH",
            "ANTOINE WARE",
            "ANTWAIN BOOKER",
            "ANTWAN WILLIAMS",
            "ANTWON DORSEY",
            "CEDRIC WILLIAMS",
            "CEDRICK DORSEY",
            "CORNELL RIVERS",
            "CORTEZ BOOKER",
            "DAQUAN WARE",
            "DARIUS BRANCH",
            "DARNELL JACKSON",
            "DARRIUS ROBINSON",
            "DASHAWN CHARLES",
            "DAVION ROBINSON",
            "DAVON RIVERS",
            "DAVONTE CHARLES",
            "DEANDRE BRANCH",
            "DEANGELO WASHINGTON",
            "DEDRICK RIVERS",
            "DEMARCUS MACK",
            "DEMARIO SINGLETON",
            "DEMETRIUS GAINES",
            "DEMOND MACK",
            "DENZEL GAINES",
            "DEONTE COLEMAN",
            "DEQUAN WARE",
            "DESHAUN SINGLETON",
            "DESHAWN BOOKER",
            "DEVANTE DORSEY",
            "DEVONTE DORSEY",
            "DOMINIQUE BRANCH",
            "DONNELL COLEMAN",
            "DONTA WASHINGTON",
            "DONTAE SINGLETON",
            "DONTE GAINES",
            "HAKEEM MOSLEY",
            "ISHMAEL CHARLES",
            "JABARI BANKS",
            "JAHEIM WILLIAMS",
            "JALEEL GAINES",
            "JAMAAL GAINES",
            "JAMAL JEFFERSON",
            "JAMAR BOOKER",
            "JAMARI JEFFERSON",
            "JAMEL MOSLEY",
            "JAQUAN RIVERS",
            "JAVON COLEMAN",
            "JAYLEN WASHINGTON",
            "JERMAINE MACK",
            "JEVON COLEMAN",
            "JUWAN GAINES",
            "KAREEM GAINES",
            "KEON WASHINGTON",
            "KESHAWN SINGLETON",
            "KEVON JOSEPH",
            "KEYON JEFFERSON",
            "KWAME MOSLEY",
            "LAMONT ROBINSON",
            "MALIK ROBINSON",
            "MARQUES BOOKER",
            "MARQUEZ WILLIAMS",
            "MARQUIS WILLIAMS",
            "MARQUISE CHARLES",
            "MEKHI BRANCH",
            "MONTRELL DORSEY",
            "OCTAVIUS WARE",
            "OMARI ROBERSON",
            "PRINCE COLEMAN",
            "RAEKWON HARRIS",
            "RAHEEM DORSEY",
            "RAQUAN BRANCH",
            "RASHAAD RIVERS",
            "RASHAD SINGLETON",
            "RASHAUN GAINES",
            "RASHAWN JEFFERSON",
            "RASHEED ROBINSON",
            "RICO GAINES",
            "ROOSEVELT ROBERSON",
            "SAVION MOSLEY",
            "SHAMAR SINGLETON",
            "SHAQUAN BRANCH",
            "SHAQUILLE WASHINGTON",
            "STEPHON BRANCH",
            "SYLVESTER DORSEY",
            "TEVIN ROBINSON",
            "TRAVON WILLIAMS",
            "TREMAINE COLEMAN",
            "TREMAYNE JACKSON",
            "TREVON JOSEPH",
            "TYQUAN JOSEPH",
            "TYREE COLEMAN",
            "TYREK COLEMAN",
            "TYRELL JOSEPH",
            "TYRESE JOSEPH",
            "TYRONE ROBINSON",
            "TYSHAWN MOSLEY",
        ],
        "ASIAN": [
            "AARON YU",
            "ADAM PHAM",
            "ADITYA NGUYEN",
            "ALAN ZHANG",
            "ALBERT CHOI",
            "ALEX ZHANG",
            "ALEXANDER CHEN",
            "AMIT KAUR",
            "ANDREW VANG",
            "ANDY HUANG",
            "ANIL SINGH",
            "ANTHONY VU",
            "ARJUN SINGH",
            "AUSTIN PHAM",
            "BENJAMIN VANG",
            "BRANDON YANG",
            "BRIAN CHOI",
            "BRYAN TRAN",
            "CHARLES LU",
            "CHRISTIAN CHOI",
            "CHRISTOPHER YU",
            "DANIEL ZHANG",
            "DANNY YANG",
            "DAVID TRAN",
            "DENNIS YU",
            "EDWARD VU",
            "ERIC LI",
            "ETHAN TRUONG",
            "EVAN ZHANG",
            "GEORGE PHAM",
            "HAI HUYNH",
            "HENRY HUYNH",
            "HUNG PHAM",
            "HUY VU",
            "IAN TRAN",
            "JACOB CHOI",
            "JAMES TRUONG",
            "JASON TRUONG",
            "JAY WU",
            "JEFFREY VU",
            "JERRY VO",
            "JIMMY CHEN",
            "JOHN TRAN",
            "JOHNNY VO",
            "JONATHAN CHEN",
            "JOSEPH PHAN",
            "JOSHUA PHAM",
            "JUSTIN WU",
            "KENNETH PHAN",
            "KEVIN YANG",
            "KRISHNA PATEL",
            "KYLE PHAM",
            "LONG HUYNH",
            "MANOJ PATEL",
            "MARK HUANG",
            "MATTHEW TRUONG",
            "MICHAEL YANG",
            "MINH LIN",
            "MOHAMMAD SINGH",
            "MOHAMMED KAUR",
            "MUHAMMAD PATEL",
            "NATHAN LU",
            "NEIL PHAM",
            "NICHOLAS PHAN",
            "NIKHIL SINGH",
            "PATRICK LI",
            "PAUL WU",
            "PETER TRUONG",
            "RAHUL PATEL",
            "RAJESH KAUR",
            "RAMESH SINGH",
            "RAVI PATEL",
            "RAYMOND VO",
            "RICHARD VO",
            "ROBERT CHEN",
            "ROHAN SINGH",
            "RYAN HUANG",
            "SAMUEL TRUONG",
            "SANDEEP SINGH",
            "SANJAY YANG",
            "SEAN HUANG",
            "SRINIVAS PATEL",
            "STEPHEN TRUONG",
            "STEVE TRUONG",
            "STEVEN ZHANG",
            "SUNIL SINGH",
            "SURESH PHAN",
            "SYED KAUR",
            "THANH YU",
            "THOMAS TRAN",
            "TIMOTHY WU",
            "TONY LU",
            "TUAN LU",
            "TYLER LU",
            "VICTOR LI",
            "VIJAY PATEL",
            "VINCENT HUYNH",
            "VINH LU",
            "VIVEK PATEL",
            "WILLIAM HUANG",
        ],
        "HISPANIC": [
            "ADRIAN BARAJAS",
            "ALAN MAGANA",
            "ALBERTO ESQUIVEL",
            "ALEJANDRO JUAREZ",
            "ALEX BARAJAS",
            "ALEXANDER HERNANDEZ",
            "ALEXIS BARAJAS",
            "ALFREDO ROSAS",
            "ANDRES GONZALEZ",
            "ANDREW JUAREZ",
            "ANGEL ROSAS",
            "ANTHONY MAGANA",
            "ANTONIO OROZCO",
            "ARMANDO BARAJAS",
            "ARTURO MEZA",
            "BENJAMIN ROSAS",
            "BRANDON GONZALEZ",
            "BRIAN IBARRA",
            "BRYAN PORTILLO",
            "CARLOS VELAZQUEZ",
            "CESAR HUERTA",
            "CHRISTIAN VAZQUEZ",
            "CHRISTOPHER AVALOS",
            "CRISTIAN HUERTA",
            "DANIEL VELAZQUEZ",
            "DAVID VAZQUEZ",
            "DIEGO VELAZQUEZ",
            "EDGAR ZAVALA",
            "EDUARDO ZAVALA",
            "EDWARD ESPINOZA",
            "EDWIN HUERTA",
            "EMMANUEL TREJO",
            "ENRIQUE ESQUIVEL",
            "ERIC ESPINOZA",
            "ERICK ESPINOZA",
            "ERIK ROSAS",
            "FELIX IBARRA",
            "FERNANDO ZAVALA",
            "FRANCISCO ESQUIVEL",
            "GABRIEL HUERTA",
            "GERARDO VAZQUEZ",
            "GUILLERMO HERNANDEZ",
            "GUSTAVO GONZALEZ",
            "HECTOR ESQUIVEL",
            "HENRY ZAVALA",
            "HUGO VILLALOBOS",
            "ISAAC ESQUIVEL",
            "ISMAEL VELAZQUEZ",
            "ISRAEL CERVANTES",
            "IVAN BARAJAS",
            "JACOB PORTILLO",
            "JAIME GONZALEZ",
            "JAMES TREJO",
            "JASON MAGANA",
            "JAVIER TREJO",
            "JESUS ZAVALA",
            "JOEL JUAREZ",
            "JOHN BARAJAS",
            "JONATHAN ESPINOZA",
            "JORGE ESQUIVEL",
            "JOSE GONZALEZ",
            "JOSEPH CERVANTES",
            "JOSHUA IBARRA",
            "JOSUE AVALOS",
            "JUAN OROZCO",
            "JULIAN CERVANTES",
            "JULIO VILLALOBOS",
            "JUSTIN VILLALOBOS",
            "KEVIN JUAREZ",
            "LEONARDO BARAJAS",
            "LUIS CERVANTES",
            "MANUEL HERNANDEZ",
            "MARCO PORTILLO",
            "MARCOS HUERTA",
            "MARIO HERNANDEZ",
            "MARTIN JUAREZ",
            "MATTHEW MEZA",
            "MICHAEL CERVANTES",
            "MIGUEL MEZA",
            "NELSON ROSAS",
            "NICHOLAS BARAJAS",
            "OMAR VELAZQUEZ",
            "OSCAR ESPINOZA",
            "PABLO HUERTA",
            "PEDRO VAZQUEZ",
            "RAFAEL VAZQUEZ",
            "RAMON VILLALOBOS",
            "RAUL PORTILLO",
            "RICARDO VELAZQUEZ",
            "RICHARD VILLALOBOS",
            "ROBERT TREJO",
            "ROBERTO CERVANTES",
            "RUBEN IBARRA",
            "SAMUEL VELAZQUEZ",
            "SANTIAGO VELAZQUEZ",
            "SEBASTIAN VAZQUEZ",
            "SERGIO HUERTA",
            "STEVEN HERNANDEZ",
            "VICTOR TREJO",
            "WILLIAM ZAVALA",
        ],
    },
    "FEMALE": {
        "WHITE": [
            "ABBY SCHMIDT",
            "ABIGAIL SCHWARTZ",
            "AIMEE NOVAK",
            "ALEXANDRA KRUEGER",
            "ALISON SCHMIDT",
            "ALLISON SCHROEDER",
            "ALLYSON OCONNELL",
            "AMANDA ERICKSON",
            "AMY KRAMER",
            "ANN YODER",
            "ANNA HUBER",
            "ANNE YODER",
            "ASHLYN KRUEGER",
            "BAILEY KOCH",
            "BETH REILLY",
            "BETHANY ERICKSON",
            "BONNIE SCHWARTZ",
            "BROOKE MUELLER",
            "CAITLIN MUELLER",
            "CAITLYN YODER",
            "CARA KRAMER",
            "CARLY REILLY",
            "CAROLINE FRIEDMAN",
            "CASEY SCHROEDER",
            "CASSIDY SCHROEDER",
            "CASSIE KLEIN",
            "CLAIRE SCHMITT",
            "COLLEEN FRIEDMAN",
            "ELISABETH SCHMITT",
            "ELIZABETH SCHNEIDER",
            "ELLEN BAUER",
            "EMILY KOCH",
            "EMMA ERICKSON",
            "ERIN SCHROEDER",
            "GINGER BAUER",
            "HAILEY YODER",
            "HALEY BAUER",
            "HANNAH SCHMIDT",
            "HAYLEY KRUEGER",
            "HEATHER ERICKSON",
            "HEIDI REILLY",
            "HOLLY KRAMER",
            "JACLYN REILLY",
            "JAIME NOVAK",
            "JEANNE REILLY",
            "JENNA KLEIN",
            "JENNIFER SCHAEFER",
            "JILL NOVAK",
            "JODI KLEIN",
            "JULIE YODER",
            "KAITLIN SCHROEDER",
            "KAITLYN KRUEGER",
            "KARA KRUEGER",
            "KARI ERICKSON",
            "KASEY NOVAK",
            "KATELYN SCHMITT",
            "KATHERINE SCHMIDT",
            "KATHLEEN SCHROEDER",
            "KATHRYN BAUER",
            "KATIE SCHWARTZ",
            "KAYLEE MUELLER",
            "KELLEY SCHNEIDER",
            "KELLIE MUELLER",
            "KELLY WEISS",
            "KELSEY HUBER",
            "KERRY BAUER",
            "KRISTA SCHNEIDER",
            "KRISTEN KLEIN",
            "KRISTI SCHMIDT",
            "KRISTIN KRAMER",
            "KRISTINE KRAMER",
            "KYLIE SCHMITT",
            "LAURA SCHAEFER",
            "LAUREN WEISS",
            "LAURIE KLEIN",
            "LEIGH WEISS",
            "LINDSAY SCHMIDT",
            "LINDSEY REILLY",
            "LORI SCHMITT",
            "LYNN REILLY",
            "MACKENZIE HUBER",
            "MADELINE KOCH",
            "MADISON SCHNEIDER",
            "MALLORY KRUEGER",
            "MAUREEN SCHAEFER",
            "MEAGAN KLEIN",
            "MEGAN KLEIN",
            "MEGHAN SCHWARTZ",
            "MEREDITH SCHNEIDER",
            "MISTY SCHMITT",
            "MOLLY OCONNELL",
            "PAIGE SCHMIDT",
            "RACHAEL SCHAEFER",
            "REBECCA SCHMIDT",
            "REBEKAH SCHWARTZ",
            "SARA KRUEGER",
            "SARAH MUELLER",
            "SAVANNAH KLEIN",
            "SUSAN HUBER",
            "SUZANNE ERICKSON",
        ],
        "BLACK": [
            "ALFREDA BRANCH",
            "AMARI ROBINSON",
            "ANIYA CHARLES",
            "ANIYAH RIVERS",
            "ARETHA WILLIAMS",
            "ASHANTI MACK",
            "AYANA HARRIS",
            "AYANNA SINGLETON",
            "CHIQUITA BANKS",
            "DASIA HARRIS",
            "DEASIA BRANCH",
            "DEJA CHARLES",
            "DEMETRIA SINGLETON",
            "DEMETRICE DORSEY",
            "DENISHA WILLIAMS",
            "DOMONIQUE BOOKER",
            "EBONI ROBINSON",
            "EBONY MACK",
            "ESSENCE JEFFERSON",
            "IESHA CHARLES",
            "IMANI CHARLES",
            "JALEESA JOSEPH",
            "JALISA HARRIS",
            "JANIYA ROBERSON",
            "KENISHA BRANCH",
            "KENYA MACK",
            "KENYATTA WARE",
            "KENYETTA DORSEY",
            "KEOSHA MACK",
            "KEYONA SINGLETON",
            "KHADIJAH CHARLES",
            "LAKEISHA ROBERSON",
            "LAKESHA ROBINSON",
            "LAKESHIA BRANCH",
            "LAKISHA DORSEY",
            "LAQUISHA JACKSON",
            "LAQUITA MOSLEY",
            "LASHANDA MOSLEY",
            "LASHAWN WARE",
            "LASHONDA WILLIAMS",
            "LATANYA ROBERSON",
            "LATASHA HARRIS",
            "LATESHA RIVERS",
            "LATISHA ROBINSON",
            "LATONIA WARE",
            "LATONYA JACKSON",
            "LATORIA GAINES",
            "LATOSHA GAINES",
            "LATOYA COLEMAN",
            "LATRICE MOSLEY",
            "MAHOGANY CHARLES",
            "MARQUITA WASHINGTON",
            "NAKIA WILLIAMS",
            "NIKIA JACKSON",
            "NIYA GAINES",
            "NYASIA JOSEPH",
            "OCTAVIA WASHINGTON",
            "PRECIOUS WARE",
            "QUIANA COLEMAN",
            "RASHIDA RIVERS",
            "SADE CHARLES",
            "SHAKIRA BRANCH",
            "SHALONDA GAINES",
            "SHAMEKA DORSEY",
            "SHAMIKA ROBINSON",
            "SHANEKA MOSLEY",
            "SHANEQUA WASHINGTON",
            "SHANICE JOSEPH",
            "SHANIKA ROBERSON",
            "SHANIQUA RIVERS",
            "SHANITA MOSLEY",
            "SHANIYA BOOKER",
            "SHANTE HARRIS",
            "SHAQUANA MOSLEY",
            "SHARITA BANKS",
            "SHARONDA JACKSON",
            "SHAVON MOSLEY",
            "SHAWANDA WARE",
            "SHERIKA ROBERSON",
            "SHERITA GAINES",
            "TAMEKA DORSEY",
            "TAMIA MOSLEY",
            "TAMIKA JEFFERSON",
            "TANESHA GAINES",
            "TANIKA JACKSON",
            "TANISHA MOSLEY",
            "TARSHA ROBINSON",
            "TAWANDA JOSEPH",
            "TAWANNA JACKSON",
            "TENISHA CHARLES",
            "THOMASINA RIVERS",
            "TIERRA MACK",
            "TOMEKA GAINES",
            "TOMIKA JACKSON",
            "TOWANDA MACK",
            "TOYA BANKS",
            "TYESHA ROBERSON",
            "UNIQUE HARRIS",
            "WILLIE WASHINGTON",
            "ZARIA JOSEPH",
        ],
        "ASIAN": [
            "ABIGAIL LI",
            "ALICE CHOI",
            "ALYSSA HUANG",
            "AMANDA LU",
            "AMY VU",
            "ANGELA VANG",
            "ANITA YU",
            "ANJALI PATEL",
            "ANNA PHAM",
            "ANNIE LI",
            "ASHLEY VU",
            "CAROLINE TRUONG",
            "CATHERINE VANG",
            "CHRISTINA PHAM",
            "CHRISTINE YANG",
            "CINDY TRUONG",
            "CLAIRE YU",
            "CRYSTAL YU",
            "CYNTHIA TRAN",
            "DIANA VU",
            "DIVYA PATEL",
            "ELIZABETH VANG",
            "EMILY YANG",
            "EMMA LU",
            "ESTHER WU",
            "GRACE VO",
            "HANNAH NGUYEN",
            "HELEN HUYNH",
            "HONG CHOI",
            "HUONG TRAN",
            "IRENE LIN",
            "JACQUELINE VU",
            "JANE LIN",
            "JASMINE LI",
            "JENNIFER PHAM",
            "JENNY HUANG",
            "JESSICA VANG",
            "JOY PHAN",
            "JOYCE LI",
            "JULIA LU",
            "JULIE HUYNH",
            "KAREN WU",
            "KATHERINE HUANG",
            "KATHY VO",
            "KAYLA VO",
            "KELLY VU",
            "KIM TRUONG",
            "KIMBERLY VO",
            "LAKSHMI KAUR",
            "LAURA PHAN",
            "LAUREN XIONG",
            "LI WU",
            "LILY LU",
            "LINDA VANG",
            "LINH PHAN",
            "LISA VU",
            "MAI CHOI",
            "MARGARET PHAN",
            "MARIA TRUONG",
            "MARY LIN",
            "MAY ZHANG",
            "MAYA YANG",
            "MEENA KAUR",
            "MEGAN YANG",
            "MELISSA YANG",
            "MICHELLE LI",
            "MONICA XIONG",
            "NANCY LI",
            "NATALIE VU",
            "NEHA PATEL",
            "NICOLE ZHANG",
            "NISHA SINGH",
            "OLIVIA PHAN",
            "PHUONG LI",
            "POOJA KAUR",
            "PRIYA SINGH",
            "PRIYANKA SINGH",
            "RACHEL ZHANG",
            "REBECCA VO",
            "SAMANTHA CHOI",
            "SARA WU",
            "SARAH CHEN",
            "SHARON LI",
            "SHIVANI CHOI",
            "SHREYA KAUR",
            "SOPHIA VO",
            "STEPHANIE XIONG",
            "SUSAN CHOI",
            "THANH LIN",
            "THAO LIN",
            "THU CHEN",
            "THUY TRUONG",
            "TIFFANY LI",
            "TINA HUYNH",
            "TRANG PHAN",
            "VAN LU",
            "VICTORIA TRUONG",
            "VIVIAN PHAM",
            "YAN YU",
            "YING PHAM",
        ],
        "HISPANIC": [
            "ABIGAIL ZAVALA",
            "ADRIANA MAGANA",
            "ALEJANDRA MEZA",
            "ALEXANDRA CERVANTES",
            "ALEXIS HERNANDEZ",
            "ALICIA HERNANDEZ",
            "ALMA BARAJAS",
            "ALONDRA GONZALEZ",
            "AMANDA GONZALEZ",
            "AMY IBARRA",
            "ANA VILLALOBOS",
            "ANDREA ZAVALA",
            "ANGELA CERVANTES",
            "ANGELICA IBARRA",
            "ANNA VAZQUEZ",
            "ASHLEY VELAZQUEZ",
            "BLANCA IBARRA",
            "BRENDA TREJO",
            "CARLA JUAREZ",
            "CARMEN MEZA",
            "CAROLINA ZAVALA",
            "CECILIA ZAVALA",
            "CHRISTINA VAZQUEZ",
            "CINDY VELAZQUEZ",
            "CLAUDIA ESPINOZA",
            "CRISTINA JUAREZ",
            "CRYSTAL VILLALOBOS",
            "CYNTHIA GONZALEZ",
            "DAISY MEZA",
            "DANIELA GONZALEZ",
            "DIANA ESPINOZA",
            "ELIZABETH VILLALOBOS",
            "EMILY ZAVALA",
            "ERICA HUERTA",
            "ERIKA TREJO",
            "ESMERALDA IBARRA",
            "EVELYN PORTILLO",
            "GABRIELA HUERTA",
            "GLORIA ESPINOZA",
            "GUADALUPE VELAZQUEZ",
            "ISABEL HUERTA",
            "ISABELLA VILLALOBOS",
            "JACQUELINE ESQUIVEL",
            "JASMINE ESPINOZA",
            "JENNIFER ESQUIVEL",
            "JESSICA TREJO",
            "JOCELYN ESPINOZA",
            "JUANA CERVANTES",
            "JULIA ROSAS",
            "KAREN BARAJAS",
            "KARINA TREJO",
            "KARLA VELAZQUEZ",
            "KATHERINE HERNANDEZ",
            "KAYLA VELAZQUEZ",
            "KIMBERLY ZAVALA",
            "LAURA VELAZQUEZ",
            "LESLIE MEZA",
            "LILIANA ZAVALA",
            "LISA JUAREZ",
            "LUZ MEZA",
            "MARGARITA IBARRA",
            "MARIA MEZA",
            "MARIBEL AVALOS",
            "MARISOL IBARRA",
            "MARITZA HUERTA",
            "MARTHA MAGANA",
            "MARY OROZCO",
            "MAYRA AVALOS",
            "MELANIE ROSAS",
            "MELISSA MAGANA",
            "MICHELLE VILLALOBOS",
            "MIRIAM MEZA",
            "MONICA AVALOS",
            "NANCY VAZQUEZ",
            "NATALIA ESPINOZA",
            "NATALIE VAZQUEZ",
            "NICOLE JUAREZ",
            "NORMA VAZQUEZ",
            "PAOLA VELAZQUEZ",
            "PATRICIA JUAREZ",
            "RACHEL HERNANDEZ",
            "RAQUEL HERNANDEZ",
            "REBECCA ZAVALA",
            "ROSA ESQUIVEL",
            "RUTH OROZCO",
            "SAMANTHA IBARRA",
            "SANDRA MEZA",
            "SARA ROSAS",
            "SARAH ESQUIVEL",
            "SILVIA OROZCO",
            "SOFIA MEZA",
            "SONIA ROSAS",
            "STEPHANIE JUAREZ",
            "TERESA OROZCO",
            "VANESSA BARAJAS",
            "VERONICA BARAJAS",
            "VICTORIA MAGANA",
            "WENDY JUAREZ",
            "YESENIA TREJO",
            "YOLANDA VAZQUEZ",
        ],
    },
}


@lambda _: _()
def names_to_demo():
    return {
        name.title(): (gender.lower(), race.lower())
        for gender, race_dict in _names.items()
        for race, name_list in race_dict.items()
        for name in name_list
    }
