jobs = {
    "carpenter": 2,
    "mechanician": 4,
    "construction worker": 4,
    "laborer": 4,
    "driver": 6,
    "sheriff": 14,
    "mover": 18,
    "developer": 20,
    "farmer": 22,
    "guard": 22,
    "chief": 27,
    "janitor": 34,
    "lawyer": 35,
    "cook": 38,
    "physician": 38,
    "ceo": 39,
    "analyst": 41,
    "manager": 43,
    "supervisor": 44,
    "salesperson": 48,
    "editor": 52,
    "designer": 54,
    "accountant": 61,
    "auditor": 61,
    "writer": 63,
    "baker": 65,
    "clerk": 72,
    "cashier": 73,
    "counselor": 73,
    "attendant": 76,
    "teacher": 78,
    "sewer": 80,
    "librarian": 84,
    "assistant": 85,
    "cleaner": 89,
    "housekeeper": 89,
    "nurse": 90,
    "receptionist": 90,
    "hairdresser": 92,
    "secretary": 95,
}
